from enum import Enum

class CVERef:
    def __init__(self, refid='',text='', url=''):
        self.refid = refid
        self.text = text
        self.url = url

class CVE:
    is_selected = False

    def __init__(self, id='', description='', targets=[], year=0):
        self.id = id
        self.description = description
        self.associated_targets = targets
        self.references = []
        self.year = year

    def __str__(self):
        return self.id

    def select(self):
        self.is_selected = True

class TargetType(Enum):
    Application = 0,
    OS = 1,
    Hardware = 2,
    NoData = 3

class Target:
    def __init__(self,vendor='',product='',type=TargetType.NoData):
        self.type = type
        self.vendor = vendor
        self.product = product

class TargetClass:
    def __init__(self,type=TargetType.NoData):
        self.type = type
        self.products = []

    def add_product(self, p):
        if p in self.products:
            return
        self.products.append(p)

    def sort(self):
        self.products.sort(reverse=True)

class Product:
    def __init__(self, name='', vendor=''):
        self.product_name = name
        self.vendor_name = vendor
        self.num = 1
        self.cves = []

    def __str__(self):
        return self.vendor_name + '_' + self.product_name

    def __eq__(self, other):
        return self.product_name == other.product_name and self.vendor_name == other.vendor_name

    def __radd__(self, other):
        return self.num + other

    def __add__(self, other):
        return self.num + other.num

    def __lt__(self, other):
        return self.num < other.num

    def ___le__(self, other):
        return self.num <= other.num

    def __gt__(self, other):
        return self.num > other.num

    def __ge__(self, other):
        return self.num >= other.num

SEPARATOR = '\\'
MAX_CVE_NUM = 35
MIN_AFFECTED_TARGET = 1