import re

import taxonomy_config as config


has_effect = "(\ allow|(which|wich)\ ((might )|)(make(s|)|may)|\ cause\ ).*"

dos_regex = "((?<!originally reported as a)(?<!for requests that cause a) denial\ ((of\ )|)service)|(affect availability)"


execution_regex = '((ex(e|)cut(e|ion)|inject)\ (.)*?(\ |)(command|code|file|expression|program|function))' \
                  '|(inject .*? script)|(code execution)|( unload kernel extension)' \
                  '|(trigger continued resolvability)|(launch arbitrary custom web applications)'

#integrity
modification_regex = "( forge )|( modify(,|) )|(add a license)|( spoof )|((?<!pointer)(?<!opcodes that) overwrite)|" \
                     "( re-encrypt )|(send unencrypted traffic)|(inject crafted packet)|(install trojan horse)|" \
                     "( change )|(upload (files|(malformed firmware)))|( delete )|( alter )|(reprogram the firmware)" \
                     "|( create (?!network))|( add )|((?<!device) reset )|( hijack )|(establish arbitrary port mapping)" \
                     "|( redirect (users|traffic))|(perform certain configuration changes)|(version downgrade)|" \
                     "( generate (?!network))|(conduct man-in-the-middle)|(hide the sender field)|" \
                     "(inject arbitrary http header)|(perform configuration change)|(spend bitcoin)|( remove )|" \
                     "(install arbitrary firmware)|(enable date and time)|( replace )|(upload software)|" \
                     "(disable packet-level encryption)|(trigger unintended use)|(inject arbitrary e-mail headers)|" \
                     "(trigger authentication-state modification)|(inject conversation text)|(conduct route-injection attacks)" \
                     "|(interfere with traffic control)|(trigger acceptance of an invalid message)|(write to files)|" \
                     "(trigger redirects)|(determine cleartext credential)"

#info leak
obtain_regex = "(obtain ((the (ip|ios|((decryption )|)key))|((a|the (administrative|logon)) password)|(server memory contents)))" \
               "|((read|obtain) (private|(arbitrary files)|username))|(obtain current login status)|(obtain .*? credentials)" \
               "|(steal)|(determine valid groupnames)|(obtain ((authentication credentials)|(the search history)))" \
               "|( decrypt )|( eavesdrop )|(to read)|(discover encrypted password)|(upload arbitrary file)|" \
               "((?<!shell\ to\ )(gain|obtain) .*?(information|username))|(defeat cryptographic protection)|( watch )" \
               "|(intercept communication)|(obtain source code)|(obtain network organization data)|(guess the key)" \
               "|(discover ((.*? password)|(wireless lan controller mac addresses)))|(determine this key)|" \
               "(obtain cleartext data)|(obtain credentials)|" \
               "(crack|guess password)|(internal information)|(confirm that a sender and receiver)|(obtain ecdsa nonces)|" \
               "(view phonebook)|( download (arbitrary|private|(a file)|))|( enumerate )|((?<!todo) list )|(capture message)|" \
               "(obtain authentication data )|(determine the device address)|(capture this cookie)|(obtain the password)" \
               "|(obtain ((cleartext contents)|(location info)))|(obtain the source code)|(extract the password)|" \
               "(obtain the administrator password)|(discover administrator credentials)|(obtain the administrator user name)|" \
               "(calculate credentials)|(obtain the pppoe/pppoa password)|(obtain the cleartext content)|(obtain the administrator's md5 password hash)" \
               "|(discover (credential|password))|(discover the original mac address)|(discover registration id)|(obtain the installation path)|" \
               "(obtain esn and min values)|(obtain the wpa or wpa2 pre-shared key)|(obtain unspecified login access)|" \
               "(trigger the omission of certain user-interface data)|(visit arbitrary resources)|(obtain the plaintext database password)"

#illegitimate access
bypass_regex = "( bypass(ing|) )|(impersonate)|(access (the|an))|(read and (?!modify))|(escape out of)|(send snmp requests)|" \
               "(obtain administrative control)|(obtain a privileged terminal)|(obtain failover-unit access)|(obtain helpdesk-equivalent privilege)|" \
               "(obtain linux root access)|(obtain application-file access)|(obtain network device administrator privileges)|" \
               "((gain|obtain)\ " \
               "(((\"|)un(authorized|specified) )|(lo(gin|cal) )|(((full )|)administrat(or|ive) )|(root )|(network )|" \
                            "(the )|(additional )|(full )|(read or write )|(system )|(linux shell )|)" \
               "((((operating-)|)system )|(console )|)(access|privilege))|(perform unspecified function)" \
               "|(establish (ssh|telnet|(an http)) connection)|(visit arbitrary resources)" \
               "|(access (arbitrary|restricted|sensitive))|(use the ftp printing interface)|(send arbitrary)|" \
               "(perform ((certain )|)actions as administrators)|(connect to the web interface)|( include )" \
               "|(perform ((arbitrary )|)(unauthorized|privileged|administrative) actions)|(grants all local users the privileges of)|" \
               "(obtain management control)|(pass the captcha)|(make arbitrary use of)|(make a(rbitrary|) phone call)|" \
               "(remove the lock)|(access memory)|(conduct unauthorized activities)|(perform mouse operations)|" \
               "(obtain database administrative access)|(obtain unspecified device access)|(accessing it)|(obtain plaintext messages)|" \
               "(obtain index\.asp login access)|(obtain superadmin access)|(gain role privileges)|(obtain cleartext password)|" \
               "(obtrain passwords)|(establish undetected concurrent login)|(obtain shell|vem access)|(gain database privileges)|" \
               "(access unintended filesystem location)|(that should not be accessible)|(obtain unintended write access)"

financial = "(calls to arbitrary telephone numbers)|(place a call without user approval)|(make telephone calls)|" \
            "(launch unintended calls)"

degrage = "(evade detection)|(use a weaker protocol)|(escape detection)|(network exposure was unintended)|" \
          "(disable wi-fi encryption)|(plug ports into the routers)"

other = "(proxy arbitrary tcp connections)|(redirect traffic)|(trick a user into loading)|" \
        "(trick users into communicating over an unintended network)|(redirect calls)|" \
        "(forwarding of vpn traffic to an attacker-controlled destination)|clickjacking|(force the user to)"

# TODO FINANCIAL

unknown = "(unknown|unspecified) impact"

known_unknown_cves = [
    'CVE-2005-0618', 'CVE-2006-6717', 'CVE-2007-2040', 'CVE-2007-3465', 'CVE-2007-3759', 'CVE-2007-4098',
    'CVE-2007-4388', 'CVE-2010-4435', 'CVE-2011-1165', 'CVE-2014-6598'
]

problematic_cves = [ # traffic redirection, weaker protocol usage, disabled IPS signature, proxying traffic,
                     # evade detection of HTTP traffic, access to interface that should not have been exposed,
                     # disable encryption, pass the CAPTCHA, trick a user into (web), force the iPhone user
    '''
    'CVE-2007-2688',
    'CVE-2007-5134', 'CVE-2008-1269', 'CVE-2008-2020', 'CVE-2008-5825', 'CVE-2007-3755', 'CVE-2007-3757',
    'CVE-2009-0961', 'CVE-2010-1756', 'CVE-2010-1810', 'CVE-2012-0333', 'CVE-2012-5032', 'CVE-2013-5482',
    'CVE-2013-5523', 'CVE-2013-5726', 'CVE-2013-6698', 'CVE-2014-0056', 'CVE-2014-2147', 'CVE-2014-3823',
    'CVE-2014-4367'
    '''
]

def categorize_effect(cve):
    result = []
    p = re.search(has_effect,cve.description)

    if p:
        if re.search(bypass_regex, p.group(0)):
            result.append(config.Effect.Illegitimate_Access)
        if re.search(obtain_regex, p.group(0)):
            result.append(config.Effect.Information_Leakage)
        if re.search(modification_regex, p.group(0)):
            result.append(config.Effect.Integrity_Violation)
        if re.search(execution_regex, p.group(0)):
            result.append(config.Effect.Code_Execution)
        if re.search(financial, p.group(0)):
            result.append(config.Effect.Financial_Loss)
        if re.search(degrage, p.group(0)):
            result.append(config.Effect.Degrading_Level_of_Protection)
        if re.search(other, p.group(0)):
            result.append(config.Effect.Other)
        if re.search(dos_regex, p.group(0)) and len(result) == 0:
            result.append(config.Effect.DoS)
        if len(result) == 0:
            if cve.id in known_unknown_cves or cve.id in problematic_cves or re.search(unknown,  p.group(0)):
                result.append(config.Effect.Unknown)
                return result
            print cve.id, cve.description
            exit(-1)
    elif re.search(unknown, cve.description):
        result.append(config.Effect.Unknown)
        return result
    elif re.search(bypass_regex, cve.description):
        result.append(config.Effect.Illegitimate_Access)
        return result
    else:
        if cve.id in known_unknown_cves or cve.id in problematic_cves:
            result.append(config.Effect.Unknown)
            return result
        print cve.id, cve.description
        exit(-1)
    result = list(set(result))
    return result

