from random import random
import pickle
from os.path import exists

from cve_preparation import config
import preparation
import cve_affected_targets


def calculate_cve_num(targetclass):
    nums = list(set([p.num for p in targetclass.products]))
    nums.sort(reverse=True)
    ratios = {}
    i = 0
    index = nums.index(config.MIN_AFFECTED_TARGET if config.MIN_AFFECTED_TARGET in nums else nums[-1])
    while i <= index:
        ratios[nums[i]] = float(nums[i]) / nums[index]
        i += 1
    return ratios

def select_cves(targetclass, ratios):
    selected_cves = []
    for r in ratios:
        # i - how many CVEs have been selected from this bin
        i = 0
        targets = [p for p in targetclass.products if p.num == r]
        cves = []
        for t in targets:
            for c in t.cves:
                cves.append(c)
        # next CVE is selected only if
        #   no more than the ration to the unit (9) has been selected from this bin and
        #   overall, no more than MAX_CVE_NUM has been selected from this category
        while i < ratios[r]:
            if len(selected_cves) < config.MAX_CVE_NUM:
                rand = int(random()*(len(cves)-1))
                if not cves[rand].is_selected:
                    cves[rand].select()
                    selected_cves.append(cves[rand])
                    i += 1
            else:
                return selected_cves
    return selected_cves



def calculate_targets():
    hardwares = config.TargetClass(type=config.TargetType.Hardware)
    oses = config.TargetClass(type=config.TargetType.OS)
    applications = config.TargetClass(type=config.TargetType.Application)
    nodatas = config.TargetClass(type=config.TargetType.NoData)
    cves = preparation.parse_cve()
    for cve in cves:
        targets = cve_affected_targets.get_targets(cve.id,cve.description)
        if targets is not None:
            for target in targets:
                product = config.Product(name=target.product, vendor=target.vendor)
                product.cves.append(cve)
                if target.type == config.TargetType.Hardware:
                    if product in hardwares.products:
                        index = hardwares.products.index(product)
                        hardwares.products[index].num += 1
                        hardwares.products[index].cves.append(cve)
                    else:
                        hardwares.add_product(product)
                elif target.type == config.TargetType.OS:
                    if product in oses.products:
                        index = oses.products.index(product)
                        oses.products[index].num += 1
                        oses.products[index].cves.append(cve)
                    else:
                        oses.add_product(product)
                elif target.type == config.TargetType.Application:
                    if product in applications.products:
                        index = applications.products.index(product)
                        applications.products[index].num += 1
                        applications.products[index].cves.append(cve)
                    else:
                        applications.add_product(product)
                elif target.type == config.TargetType.NoData:
                    if product in nodatas.products:
                        index = nodatas.products.index(product)
                        nodatas.products[index].num += 1
                        nodatas.products[index].cves.append(cve)
                    else:
                        nodatas.add_product(product)
                else:
                    raise Exception('Unknown target type! CVE: ' + cve.id)
    print 'Saving...'
    hardwares.sort()
    f = open('C:\\BME\AIT\\repository\\attack taxonomy\\cve\\data\\nocisco\\hardware.dump','w')
    pickle.dump(hardwares,f)
    f.close()
    oses.sort()
    f = open('C:\\BME\AIT\\repository\\attack taxonomy\\cve\\data\\nocisco\\os.dump','w')
    pickle.dump(oses,f)
    f.close()
    applications.sort()
    f = open('C:\\BME\AIT\\repository\\attack taxonomy\\cve\\data\\nocisco\\application.dump','w')
    pickle.dump(applications,f)
    f.close()
    nodatas.sort()
    f = open('C:\\BME\AIT\\repository\\attack taxonomy\\cve\\data\\nocisco\\nodata.dump','w')
    pickle.dump(nodatas,f)
    f.close()
    print 'Saved'


if __name__ == '__main__':
    needData = False
    hardwares = config.TargetClass(type=config.TargetType.Hardware)
    oses = config.TargetClass(type=config.TargetType.OS)
    applications = config.TargetClass(type=config.TargetType.Application)
    nodatas = config.TargetClass(type=config.TargetType.NoData)
    if exists('C:\\BME\AIT\\repository\\attack taxonomy\\cve\\data\\nocisco\\hardware.dump'):
        print 'Loading Hardware...'
        f = open('C:\\BME\AIT\\repository\\attack taxonomy\\cve\\data\\nocisco\\hardware.dump','r')
        hardwares = pickle.load(f)
        f.close()
    else:
        needData = True
    if exists('C:\\BME\AIT\\repository\\attack taxonomy\\cve\\data\\nocisco\\os.dump'):
        print 'Loading OS...'
        f = open('C:\\BME\AIT\\repository\\attack taxonomy\\cve\\data\\nocisco\\os.dump','r')
        oses = pickle.load(f)
        f.close()
    else:
        needData = True
    if exists('C:\\BME\AIT\\repository\\attack taxonomy\\cve\\data\\nocisco\\hardware.dump'):
        print 'Loading Application...'
        f = open('C:\\BME\AIT\\repository\\attack taxonomy\\cve\\data\\nocisco\\application.dump','r')
        applications = pickle.load(f)
        f.close()
    else:
        needData = True
    if exists('C:\\BME\AIT\\repository\\attack taxonomy\\cve\\data\\nocisco\\nodata.dump'):
        print 'Loading NoData...'
        f = open('C:\\BME\AIT\\repository\\attack taxonomy\\cve\\data\\nocisco\\nodata.dump','r')
        nodatas = pickle.load(f)
        f.close()
    else:
        needData = True
    if needData:
        calculate_targets()
    ratio = calculate_cve_num(oses)
    cv = select_cves(oses, ratio)
    ratio = calculate_cve_num(hardwares)
    cv = cv + select_cves(hardwares, ratio)
    ratio = calculate_cve_num(applications)
    cv = cv + select_cves(applications, ratio)
    ratio = calculate_cve_num(nodatas)
    cv = cv + select_cves(nodatas, ratio)
    print len(cv)
    balanced_dataset = open('C:\\BME\AIT\\repository\\attack taxonomy\\cve\\data\\nocisco\\balanced_dataset.txt', 'w')
    for c in cv:
        balanced_dataset.write(c.id + config.SEPARATOR + c.description + '\n')
    balanced_dataset.close()
