import pickle
import config
from lxml import etree


white_list = [
    'control_systems',
    'ics-cert.us-cert.gov',
    'camera',
    'phone',
    'opc',                                  # Object Linking and Embedding (OLE) for Process Control ( standards by an industrial automation industry task force)
    'comos',                                # plant engineering software
    'ge intelligent platforms',             # data visualization fo plants
    'wincc',                                # for SCADA system
    'modem',
    'printer',
    'laserjet',
    'cobham',                               # satellite embedded systems
    'satellite',
    'scada',
    'vxworks',                              # embedded OS
    'switch',
    'juniper junos',                        # embedded OS
    'router',
    'sensor',
    'plc',
    'firmware',
    'garrettcom magnum',                    # network devices
    'ipc',                                  # industrial PC
    'htc',                                  # mobiles
    'synel',                                # data collection systems and solutions for workforce management
    'd-link',                               # company manufacturing embedded network devices
    'lattice semiconductor pac-designer',   # for working with Verilog HDLs
    'broadcom',                             # semiconductor company in the wireless and broadband communication business
    'cisco',                                # manufacturer of embedded network devices
    'ruggedcom',                            # company dealing in SCADA OS
    ' tv ',
    'sitecom',                              # manufactures device for home networks
    'schneider electric',
    'rockwell',                             # a global provider of industrial automation and information solutions
    'hp-ux',                                # OS for HP products
    'chipset',
    'blue coat'                            # provider of security and networking solutions
]

black_list = [
    'drupal',
    'adobe',
    'basic analysis and security engine (base)',
    'red hat',
    'typo3',
    'switching user',
    'cubecart',
    'server',
    'android',
    'mac os',
    'chrome',
    'mozilla',
    'opera ',
    ' wordpress ',
    'jsse',
    'it should not be included in cve'
]


def parse_cve():
    cves = []
    tree = etree.parse('C:\\BME\\AIT\\repository\\attack taxonomy\\cve\\data\\allitems.xml')
    root = tree.getroot()
    for entry in root:
        cve = config.CVE()
        cleanDesc = False
        cleanRef = False

        cve.id = entry.get('name')

        #print 'Processing ', cve.id
        should_be_processed = True
        blacklist_found = False
        cve.year = int(entry.get('seq').split('-')[0])
        if cve.year > 2004:
            for desc in entry.iterchildren('{http://cve.mitre.org/cve/downloads}desc'):
                cve.description = desc.text.encode('utf-8').lower()
                for c in white_list:
                    if c.encode('utf-8') in cve.description:
                        cleanDesc = True
                        is_needed = True
                        for a in black_list:
                            if cve.description.find(a) > -1:
                                print 'Blacklist word found: ', a
                                should_be_processed = False
                                is_needed = False
                                blacklist_found = True
                                break
                        if not is_needed:
                            cleanDesc = False
                            break

            for ref in entry.iterchildren('{http://cve.mitre.org/cve/downloads}refs'):
                for r in ref.iterchildren('{http://cve.mitre.org/cve/downloads}ref'):
                    url = r.get('url')
                    text = r.text
                    cve.references.append(config.CVERef(refid=r.get('source'),
                                                        text=text,
                                                        url=url))
                    if text is None:
                        text = ''.encode('utf-8')
                    else:
                        text = text.encode('utf-8')
                    if url is None:
                        url = ''.encode('utf-8')
                    else:
                        url = url.encode('utf-8')
                    for c in white_list:
                        if c.encode('utf-8') in text or c.encode('utf-8') in url:
                            cleanRef = True
                            is_needed = True
                            for a in black_list:
                                if a.encode('utf-8') in text or a.encode('utf-8') in url:
                                    is_needed = False
                                    break
                            if not is_needed:
                                cleanRef = False
                                break

        if (cleanDesc or cleanRef) and not cve.description.find('** reject **') > -1:
            if not blacklist_found:
                cves.append(cve)

    return cves

separator = ';;;'

if __name__ == '__main__':
    cves = parse_cve()

    cve_embedded = open('C:\\BME\AIT\\repository\\attack taxonomy\\cve\\data\\cve_embedded_from2005.csv', 'w')
    pickle.dump(cves, open('C:\\BME\AIT\\repository\\attack taxonomy\\cve\\data\\cves.dump', 'w'))
    for cve in cves:
        cve_embedded.write(cve.id + separator + cve.description + separator)
        i = 0
        for r in cve.references:
            cve_embedded.write(str(r.refid) + ':' + str(r.text) + ':' + str(r.url))
            if i != len(cve.references)-1:
                cve_embedded.write(config.SEPARATOR)
            i += 1
        cve_embedded.write('\n')
    cve_embedded.close()
