import re

import taxonomy_config as config


programming_regex = "(message parsing code)|(check the call-id, branch, and tag values)|(packet with a zero)" \
                    "|(assertion fail)|(bluetooth nickname)|((buffer|stack) overflow)|(large (number|amount) of)|" \
                    "(race condition)|(may not process certain attributes)|(context switch)|" \
                    "(((shell meta)|(control )|(certain )|(\(0xff\) ))character)|((?<!sql )injection)|(disables a verification step)|" \
                    "(out-of-bounds read)|(same source and destination ips)|(mismatched k values)|" \
                    "(the second tag to be redirected)|(where the source mac is modified)|(negative linger time)|" \
                    "(incorrect offset)|(null ip option length)|(prevents the request from being sent to websense)|" \
                    "(untrusted search path)|(leave an empty rpath or runpath)|(error message that displays)|" \
                    "(negative expires field)|(format string)|(content-length field that has more than 9 digits)|" \
                    "(missing argument)|( error message)|(mutex)|unicod|" \
                    "(only intended for testing)|(recover gracefully)|(generates different error messages)|" \
                    "(trailing dot in a request)|(parameter.*?, which is not validated)|(improperly handled error condition)" \
                    "|(parasitic public keys)|(reconfigures itself)|(file inclusion)|" \
                    "(btvoyager_decoder\.c)|((bad|invalid) checksum)|(null pointer dereference)|( flood(,|ing|) )|(eflags)|" \
                    "(sending a vtp)|((syn|port) scan)|(parameter without a value)|(modifying \(1\) the http user-agent header)" \
                    "|((memory|heap) (leak|segment|management|consumption|corruption))|(obex push a file)" \
                    "|(uses javascript delays)|(misinterprets a menu button)|(different returned message)|" \
                    "(not properly handle(d|) (?!admin))|(properly parse)|(incorrect variable)|" \
                    "(unicode character encodings)|(non-root user creates a kernel process)|(display a different number)|" \
                    "(certain ssl/tls handshake requests)|(does not disable it)|(targets the ip address)" \
                    "|(streamids from different)|asterisk|(many request)|(many lisp tcp session)|" \
                    "(without a (content-type|host) header)|(message with a via header)|(out-of-order tcp packet)|" \
                    "((integer|buffer) overflow)|(use-after-free)|(double free)|(request-uri lacks a user name)|" \
                    "(immediately subsequent sip cancel message)|(check the return value)|(invoked using invalid buffer)" \
                    "|(null dereference)|(persistence of)|(connections and disconnections)"

programming_regex2 = "(provides different responses)|(placing a character at the end)|(sets the so_reuseaddr socket option)" \
                    "(garbage collect)|(stress test)|(validate sip url)|(identify packet fragments)|" \
                    "(insufficient number of different images)|(signedness)|(validate the origin)|(uri that begins)|" \
                    "(triggers unspecified ips signatures)|(does not initialize)|uninitialized|(does not clear)|" \
                    "(overwrite(( arbitrary)|) memory)|(that occurs during termination)|(emergency(\ |-)call)|" \
                    "(does not isolate)|(tcp state table)|(does not properly display)|(logic error)|" \
                    "(data over an established)|(specifies zero bytes)|(deleting a single cell)|(many as numbers)|" \
                    "(misinterprets the data type)|(dismisses the call approval dialog)|(null checks)|" \
                    "(does not limit the rate of)|homoglyph|(generates a response if)|(disregards the access flag)|" \
                    "(does not properly validate sql command)|(making many passive ftp)|(interrupt sharing)|" \
                    "(infinite loop)|(boundary restrictions)|(out-of-bounds memory)|(interrupting the handler)|" \
                    "(large \"ip textfield\" size)|(drop packets)|(embryonic tcp connections that remain)|" \
                    "(does not close the tcp connection)|(buffer (leak|exhaustion))|(pbuf exhaustion)|" \
                    "(meeting invitation)|(remove sshd_lock files)|(read the contents of memory locations)|" \
                    "(threaded processes)|(slash slash)|(cpu consumption)|(test parameter)|(via network traffic)|" \
                    "(eigrp multicast storm)|(space character)|(tftp copy over ipv6)|(script opcode)|(html entities)|" \
                    "(recover memory)|(making many authentication)|(despite a vpnclient mode configuration)|" \
                    "(does not properly support)|(insecure exposed method)|(buffer over-read)|(segmentation fault)|" \
                    "(does not create a fragment entry)|(obtain control of a function pointer)|(core dump)|" \
                    "(making authentication attempts that exceed the configured limit)|(crafted sccp registration)|" \
                    "(writes the secure heap)|(does not recognize the vrf-also keyword)|(trigger a shun event)|" \
                    "(trigger syslog message)|(discards all lines that end with a log or time keyword)|" \
                    "(sequence of ipv6 packets)|(sequence of \(1\) http)|(sending ike udp packets over \(1\) ipv4)|" \
                    "(action to the hnap interface)|(ssdp reflection)|(uuencode encoding)|(process hang)|" \
                    "(check the mac address for a request)|(thread local storage)|(out-of-bounds array access)|" \
                    "(a \"new\" value in the isnew parameter)|(invalid pointer)|(data that is improperly handled)|" \
                    "(does not properly match ports)|(crash and restart)|(processes trailing strings)|" \
                    "(ccc_flush_sql_file opcode)|(take focus)|(environment variable)|(validate parameters)" \
                    "|(validate resource-queue metadata)"

programming_regex3 = "(do(es|) not properly (process|clear|implement))|(extended use of the product)" \
                    "|(validate ioshareddataqueue object metadata)|(kerberos ticket is in the cache)|" \
                    "(initialize((\ kernel)|) memory)|(bounds checking)|(analyze zip archives)|" \
                    "(interrupt timer collision)|(polling frequently)|(packets that require tower processing)|" \
                    "(invalid credentials)|(commands as arguments)|(file-descriptor exhaustion)|" \
                    "(series of \(1\) virtual switching systems)|(encapsulated igmp data)|(sending data to tcp port)|" \
                    "(overwrites cached server names)|(sending packets at a high rate)|(input-queue)|" \
                    "(completing local web authentication quickly)|(modified local-preference)|(invalid read operation)|" \
                    "(does not compare the timestamp of offered software)|(different amount of time for)|" \
                    "(spoke-to-spoke traffic)|(mvpnv6 update)|(certain frame relay traffic)|(sed e option)|" \
                    "(not properly monitor load balancer)|(expected packet drops do not occur)|(resource consumption)|" \
                    "(integer underflow)|(state management)|(places kernel pointers)|(not properly deallocate memory)|" \
                    "(not ensure that a set-cookie http header is complete)|(text-relocation instructions)|" \
                    "(not properly evaluate the expiration date)|(analysis engine process outage)|(using \%2f sequences)|" \
                    "(swap operations)|(data structure has been initialized)|(not properly use the stun and turn protocols)|" \
                    "(request for an unspecified mib)|(chunk corruption)|(not properly use a dhcp pool)|(rsargs parameter)" \
                    "|(not properly pad transmissions)|(request that triggers a rewrite)|(many pptp packets)|" \
                    "(many username-from-cert ike requests)|(disk consumption)|(manage the state)|" \
                    "(many media snapshot requests)|(igmp_snoop_orib_fill_source_update)|(bursty network traffic)|" \
                    "(traffic amplification)|(not properly manage the lock state)|(not properly count references)|" \
                    "(many packets to tcp port)|(stack exception)|(not properly suppress error-condition details)|" \
                    "(high rate of)|(dirty flag)|(sanitizing)|(regex hardware job failure)|(manage memory)"

programming_regex4 = "(not properly manage buffer)|(not properly validate the identity of vcenter consoles)|" \
                     "(invalid smart call home contact)|(embedding commands in an unspecified parameter)|" \
                     "(sed \(1\) r and \(2\) w commands)|(shell output redirection)|(bit in the guest cr4 register)|" \
                     "(transit ip packets)|(interface queue)|(invalid dtls handshake)|(install all iso\" arguments)|" \
                     "(crafted ipv4 dns tcp stream)|(not properly validate((\ unspecified)|) input)|(square brackets)|" \
                     "(not properly terminate session)|(many (tcp|sip) packets)|(incorrectly terminated http session)|" \
                     "(denial of service \(device reload\) via a crafted certificate)|(crafted as path set)|" \
                     "(not properly validate tcp connection information)|(not accompanied by syn-ack packets)|" \
                     "(modification of status bit)|(not properly interact with the kernel)|" \
                     "(not properly validate unspecified system call)|(freeing the page)|(repeated requests)|" \
                     "(not properly manage ssh session)|(certain buttonclicked value)|(unhandled exception)|" \
                     "(not properly validate copyin and copyout arguments)|(accesses pipe object pointers)|" \
                     "(not ensure that it has been invoked in an abort context)|" \
            "((malformed|truncated|segmented|(fragmented (esp|gre))|invalid|long|wappush|mpls|" \
            "zero(-|( in the ))length|crafted|large).*? " \
                "((?<!later )packet|command|(?<!\(1\) )update|batchbuffer|url(?!, \(2\))|((?<!unspecified )request)|" \
                "(?<!bootstrap )message|prefix|datagram|segment|method|radio|name|argument" \
                "|field|string|(session(?!, aka bug id cscuc79779))|data|descriptor|frame|" \
                "file|input|uri|attribute|value(?!, which)|username|" \
                "header|option|character|object|connection|response|(digest domain)|parameter|traffic|quer(y|ies)))"

web = "(cross-site scripting)|(cross-site request forgery)|(sql injection)|( cookie )|" \
      "(web (page|site(s|)) (with|from|via|that))|(does not prevent photo-library access)|" \
      "( redirect user)|(disable remote image loading)|(clickjacking)|(xml external entity)|" \
      "csrf(?!( attack))|(xss(?!( sequence)))|(redirect users to other websites)|(crlf)|" \
      "(inject arbitrary web script)|(template html)|(https request with 127\.0\.0\.1)|(inject web pages)|" \
      "(remote web site)|(php code)|(processed by the preg_replace function)|(multiple http request)|" \
      "(javascript code that calls)|(xml external entities)|(via vectors involving javascript)|" \
      "(insecure method vulnerability)|(activex control)|(spoof the origin website)|(modified javascript)|" \
      "(session fixation vulnerability)"

weak_ac = "(directory traversal)|(direct(( url)|) request(\.|(s|)(?!( to \(1\) thum)|(, which))))|" \
          "(temporary files insecurely)|(reuse a tcl shell process across login session)|" \
          "((store|display)(s|) .*?(cleartext|plaintext))|(without proper protection)|(without authentication)|symlink|" \
          "((default|blank) .*?(account|community|password|permission|username))|" \
          "(hard(-|)coded)|(do not require authentication)|(leave the user in the new shell)|" \
          "(does not use an authentication initialization function)|(reading a \(1\) error log)|" \
          "(using arbitrary snmp credentials)|(accepts sip invite requests from arbitrary source)|" \
          "(listens to udp port)|(sets null authentication)|(modifying the password field)|" \
          "(the privileged management gui to launch)|(world-(readable|writ(e|)able) permission)|(access to a servlet)|" \
          "(debug service on tcp port)|(running with system privileges)|(preserve acl behavior)|" \
          "(does not enforce connect rule)|(not filtered by the modem when creating)|(helomoto)|" \
          "(does not follow file and mmap restrictions)|(grants the java\.security\.allpermission permission)|" \
          "(give write permission to the attachment)|(access would be otherwise restricted)|" \
          "(verifies the users identity via http headers)|(redirecting a command\'s output)|(flag=detect request)|" \
          "(changing certain properties)|(does not record device properties)|(quarantined devices to communicate over)|" \
          "(management packets)|((read|delete) arbitrary files (?!.*?error message))|(limit dns service access)|" \
          "(spoofed from the nam's own ip)|(packet to udp port 916)|(displays the form field)|(attempted ssh session)|" \
          "(checks the client's uid on the listening af_local socket)|(modified authentication sequence)|" \
          "(operates despite the system password lock)|(do not drop privilege)|(grants all local users the privilege)" \
          "|(insecure default discretionary access control list)|(does not protect confidentiality or integrity)|" \
          "(start before logon)|(weak permission)|(certain sip invite message followed by)|(places a \"no login\" line)|" \
          "(does not correctly report password status)|(127/8 ip addresses)|(make unauthorized files accessible)" \
          "|(message that enables)|(send(s|)(?!( secure)) .*? cleartext)|(by obtaining the encrypted password)|" \
          "(enable key-based authentication)|(resubmitting the login attempt)|(packet of type \(1\) 0x2728)|" \
          "(a process that executes a command shell and listens on a randomly chosen tcp port)|(reading the pwd field)" \
          "|(authentication state by ip address)|(entering characters at the enable prompt)|(credentials in a web form)|" \
          "(accessing administrative uri)|(trusted host value)|(authentication over http via a hash string)|" \
          "(\.html, which discloses)|(minimum password length)|(assigns uid 0)|(sends configuration data in response)|" \
          "(read/write snmp service with \"private\")|(sends cleartext configuration)|(relies on client-side javascript)"

weak_ac2 = "(policy evasion)|(bypass the filters)|(auto(-|)complet(e|(ion mechanism)))|(back(\ |)door)|" \
           "(access (memory in other)|(the system controller))|(replace the password policy)|" \
           "(do(es|) not(( properly)|) (verify(?! the offset)|encrypt|" \
                    "(check the ownership)|(enforce.*? acl)|(perform(?!( (runtime|garbage|interrupt))))" \
                    "|restrict|set|authenticate|(require(?!( authentication to exploit)))))|" \
           "(setting the login parameter to admin)|(accessing certain remapping registers)|(closing the connection)|" \
           "(lists deleted e-mail messages)|(\$admin value)|(undo deletions of characters in the password)|" \
           "(does not reset supplementary group)|(applying vt-d protection)|(already exists in /var/tmp/fw_ip)|" \
           "(complete an ssl handshake with an https client even if this client is unauthorized)|" \
           "(volatile.user.administrator to true)|(exposes ip services)|(interact with the menu for snr number changes)|" \
           "(community name of public)|(discover a group password)|(permits the viewing of cifs)|" \
           "(information via a get request)|(renaming this program to inst\.exe)|(read the private key)|" \
           "(recreate\.php and then)|(without verifying its authenticity)|(does not properly validate users)|" \
           "(leveraging .*? privileges)|(default oracle database credentials)|(crafted tacacs\+ reply)|" \
           "(trojan horse gauntlet)|(encoded dot)|(calculation on the mac address)|(modbus)|" \
           "(failed pin authentication)|(create arbitrary directories)|(administrative resource)|" \
           "(help desk administrator role)|(unrestricted file upload)|(reading \(1\) files)|" \
           "(cleartext-equivalent traffic)|(request to wirelesschannelstatus.html)|((rapid|) series of)|" \
           "(modify data via a jnap action)|(guest account via a modified http request)|(reading log (files|data))|" \
           "(examining the html source code)|(establishing a telnet session during a recovery boot)|" \
           "(http handler)|(reading a temporary)|(access to the guest account)|(app in the background state)|" \
           "(leveraging membership in)|(snmp community of public)|(weak system-object access control)|" \
           "(reading the verbose response data)|(passwords in form data)|(visiting the change-password page)|" \
           "(not use encryption)|(reading((((\ \"extraneous information\" in)|)\ html)|( the)|) source code)|(acl entries via fragmented packets)|" \
           "(include a one-way hash of a password)|(password derived from the mac address)|(cookie: :language=en)|" \
           "(maximum password length of 8 bytes)|(cleartext information)|(world writable permissions)"


weak_ac3 = "(injecting a certificate trust list)|(incorrect file permission)|(uploading new firmware)|" \
           "(not properly enforce authorization)|(request to this interface)|(cmdwebgetconfiguration action)|" \
           "(reset all configuration values)|(overload\" of the command-line utility)|(changes the admin password)|" \
           "(remove the password protect)|xmlset_roodkcableoj28840ybtide|(request to photo/p/api/list\.php)|" \
           "(has default service module credentials)|(incorrect visibility of the passcode-entry)|" \
           "(configuring a source ip address that had previously)|(default wps pin)|(comos project access)|" \
           "(not use cave authentication)|(read images of arbitrary scanned documents)|" \
           "(unspecified form that is accessible)|(default wpa2 psk passphrase)|(leveraging login access)|" \
            "(leveraging access to the low-priority)|(shows the wi-fi psk passphrase in cleartext)|" \
            "(cleartext password in a config)|" \
            "(use cleartext to store sensitive)|(administrator password via a list action)|" \
            "(not properly validate credentials)|(relies on client-side privilege checks)|" \
            "(not properly determine the existence of an authenticated session)|(hidden form fields in an html document)|" \
            "(weak file permissions)|(incorrect operating-system permissions)|(when an active administrator session exists)|" \
            "(incorrect validation of the http authorization header)|(preset\" action)|(reading the \"var pass=\")|" \
            "(cleartext base64 format for storage of passwords)|(reading the \_paramr\[\"p\"\])|" \
           "(knowledge of previously valid credentials)|(attempts a cleartext login)|(cleartext credentials)|" \
           "(terminates validation of a route permission table entry)|(cleartext ipmi credentials)|" \
           "(fragmented \(1\) ipv4(?!( or \(2\) ipv6 packets, aka bug id cscue51272.)))|" \
           "(leveraging the network-operator role)|(the same privilege level for execution)|" \
           "(not properly consider the scsi device class during authorization)|(invalid cidr in a security group rule)|" \
           "(not properly manage privileges for anonymous logins)|(reading a \(1\) technical-support bundle file)|" \
           "(not properly validate binaries)|(no properly check privileges)|(not properly assign privileges)|" \
           "(relies on the ip address for authentication)|(not ensure that updates have a valid origin)|" \
           "(not properly represent webnavigator credentials)"

improper_crypto = "(appear to be too old)|(uses the same private key)|(weak (encryption|challenge))|(predictable)|" \
                  "( predict )|(non-random value)|(removes pkcs-1 padding before generating a hash)|" \
                  "(have the same shared se(rc|cr)et)|(do not validate the ssl/tls certificates)|" \
                  "(fixed des key)|(random )|(does not use the cnonce parameter)|(base64-encoded cleartext)|" \
                  "(only 65536 different values in the 32-bit id number)|(weak cryptography)|" \
                  "(relies on the client to specify the hmac length)|(insufficient randomness)|" \
                  "(stores a partition encryption key in an unencrypted /system/.crypt file with base64 encoding)|" \
                  "(do not use a salt)|(changes the encryption level)|(use(( the same)|) rsa )|" \
                  "(create an undocumented recovery key)|(plain cbc mode)|(same challenge)|(weak hashing algorithm)|" \
                  "(rand library function to generate a certain recovery key)|(validate certain certificates)|" \
                  "(same static, private ssl keys)|(adequate message-digest algorithm)|(single-iteration pbkdf2)|" \
                  "(caching of certain public keys)|(crafted challenge)|(mac address as part of the rc4 setup key)|" \
                  "(validate the subject alternative name)|(nondeterministic cbc padding)|(improper encryption)|" \
                  "(arbitrary verisign certificate)|(weak ssl encryption settings)|(improper algorithm for pin codes)" \
                  "|(reuses the rc4 cipher stream)|(time value for entropy)|(not use a salt)|(weak hash algorithm)|" \
                  "(not have sufficient entropy)|(weak rsa keys)|(not use a sufficient source of entropy)|" \
                  "(cli copy command in a local-mgmt context)|(not properly validate authentication)|" \
                  "(includes a cryptographic key in binary files)|(read the messages of arbitrary users)|" \
                  "(improperly handled during decryption)|(using cipher suite 0)|(prng algorithm)|" \
                  "(same aes encryption key)|(not properly validate x.509 certificate)|" \
                  "(crafted x.509 client certificate)|(weak password-hashing algorithm)|" \
                  "(wps pin based on 6 digits of the lan/wlan mac address)"

insecure_config = "(configured for load balancing)|(enables snmp by default)|((?<!non-)default configuration)|" \
                  "(does not properly block inbound traffic)|(send unencrypted traffic to a secure network)|" \
                  "(directly accessing udp port)|(while generating the cisco ios ips configuration file)|" \
                  "(does not allow (users|(the user)) to disable access)|(information about network nodes)|" \
                  "(signatures that use regular expressions)|(redirect traffic)|(enable(d|s|)\ .*?by default)|" \
                  "(uses a multicast group as the destination)|(does not prompt the user)|(networking changes)|" \
                  "(gain access to the vlan)|( does not properly report the wireless network that is in use)|" \
                  "(permissive whitelist)|(request(s|) to udp port)|(responds to an aggressive mode)|" \
                  "(determine the interfaces for which telnet connections should be permitted)|" \
                  "(permit packets to pass before)|(to a restricted destination)|(opens ports in upnp routers)|" \
                  "(establishing two ssh2 sessions)|(icmpv6 echo request packet containing a hop-by-hop)|" \
                  "(source port of a packet)|(insecure connection)|(external forwarding)|" \
                  "(installation path via the file parameter)|(not properly bind)|" \
                  "(changing a ppp configuration)|(arbitrary subnets)|(an ipv6 non-provider subnet)|" \
                  "(permanent device binding)|(prevent the presence of addresses)|(enables voice dial during)|" \
                  "(attaching an ipv6 private subnet to a l3 router)|(transit traffic)|(incorrect cli restriction)|" \
                  "(falls back to http)|(not properly manage sessions)|(enable a telnet)|(listening during a pstn call)|" \
                  "(ssh session to a management interface)|(places session identifiers in get requests)|" \
                  "(reading extraneous fields in an html document)|(x11uselocalhost configuration setting is disabled)|" \
                  "(not properly share a management ip address)|(ftp session that uploads)|(does not prevent replaying)|" \
                  "(in certain configurations of)|(undocumented access method)|(test interface)|(tftp encrypted config)|" \
                  "(not check the tenant id when creating ports)|(relies on the linux-ios internal-network configuration)|" \
                  "(provides the training-session url)|(unspecified interface)|(configuration with overlapping criteria)|" \
                  "(not preserve ((the source)|file) (restriction|permission))|(ipsec tunnel traffic)|" \
                  "(uses a bssid masking approach)|(netstate request)|(observing ipmi interface responses)|" \
                  "(outbound ssh session)|cscuq12117|(sniffing the network for http query data)|(crafted device)|" \
                  "(using udp ports 17336 and 17388)|(commands via a tcp replay attack)|(debug service)|" \
                  "(jumbo packet to the management interface)|(establishing an ssh session from a client)|" \
                  "(specifies a logic-execution stop and fault)|(cip message that modifies the \(1\) configuration)|" \
                  "(cip message that specifies a reset)|(packet to udp port 69)|(performs network-beacon broadcast)|" \
                  "(not properly validate link state advertisement)|(modify the device configuration via an snmp request)"

unknown = "((unknown|unspecified) vulnerabilit(y|ies))|(demonstrated by a vulnerability scan)"

known_unknown_cves = [ # both vulnerability and attack method is missing from the description
                       # if the "attack method" was a vulnerability scan, it is still unknown how the vuln can be exploited
    'CVE-2005-0835', 'CVE-2005-2862', 'CVE-2005-3564', 'CVE-2006-2355', 'CVE-2006-3344', 'CVE-2006-4015',
    'CVE-2007-1065', 'CVE-2007-1072', 'CVE-2007-4380', 'CVE-2007-4789', 'CVE-2008-0707', 'CVE-2008-2059',
    'CVE-2008-5385', 'CVE-2008-6895', 'CVE-2008-6896', 'CVE-2008-7201', 'CVE-2009-0474', 'CVE-2009-1155',
    'CVE-2009-2256', 'CVE-2010-0575', 'CVE-2010-2842', 'CVE-2010-2843', 'CVE-2010-3033', 'CVE-2010-3034',
    'CVE-2011-0951', 'CVE-2011-1372', 'CVE-2011-1472', 'CVE-2011-1602', 'CVE-2011-1603', 'CVE-2011-1645',
    'CVE-2011-2404', 'CVE-2011-2584', 'CVE-2011-3289', 'CVE-2011-4525', 'CVE-2011-4882', 'CVE-2012-0131',
    'CVE-2012-0364', 'CVE-2012-0371', 'CVE-2015-2050', 'CVE-2015-1448', 'CVE-2015-0977', 'CVE-2015-0621',
    'CVE-2015-0606', 'CVE-2015-0584', 'CVE-2014-7999', 'CVE-2014-7995', 'CVE-2014-7879', 'CVE-2014-7269',
    'CVE-2014-5231', 'CVE-2014-3895', 'CVE-2014-3816', 'CVE-2014-3342', 'CVE-2014-2126', 'CVE-2012-0384',
    'CVE-2012-1239', 'CVE-2012-1809', 'CVE-2012-2283', 'CVE-2012-2490', 'CVE-2012-3009', 'CVE-2012-3739',
    'CVE-2014-2121', 'CVE-2014-1997', 'CVE-2014-1383', 'CVE-2014-0747', 'CVE-2014-0742', 'CVE-2014-0679',
    'CVE-2014-0676', 'CVE-2014-0669', 'CVE-2014-0615', 'CVE-2013-7093', 'CVE-2013-6969', 'CVE-2013-6964',
    'CVE-2013-6840', 'CVE-2013-6015', 'CVE-2013-5533', 'CVE-2013-5506', 'CVE-2013-5493', 'CVE-2013-3708',
    'CVE-2013-3430', 'CVE-2013-3402', 'CVE-2013-3401', 'CVE-2013-2802', 'CVE-2013-2788', 'CVE-2012-4095',
    'CVE-2012-4096', 'CVE-2012-4621', 'CVE-2013-1215', 'CVE-2013-1189', 'CVE-2012-4688', 'CVE-2012-5037',
    'CVE-2013-0689', 'CVE-2013-1107'
]

problematic_cves = [ #lots of vulns related to configuration; traffic redirection?
    'CVE-2005-0328', 'CVE-2010-2981', 'CVE-2010-2983', 'CVE-2010-5189', 'CVE-2011-0395', 'CVE-2012-0133',
    'CVE-2014-6602', 'CVE-2014-2360', 'CVE-2013-1229'
]

def categorize_vulnerability(cve):
    result = []
    if re.search(unknown, cve.description) or cve.id in known_unknown_cves or cve.id in problematic_cves:
        result.append(config.Vulnerability.Unknown)
    else:
        if (re.search(programming_regex, cve.description) or re.search(programming_regex2, cve.description)
            or re.search(programming_regex3, cve.description) or re.search(programming_regex4, cve.description)) \
            and not re.search(web, cve.description) \
            and not (re.search(weak_ac, cve.description) or re.search(weak_ac2, cve.description)) \
            and not re.search(insecure_config, cve.description):
            if cve.id == 'CVE-2013-2819':
                print re.search(programming_regex, cve.description)
                print re.search(programming_regex2, cve.description)
                print re.search(programming_regex3, cve.description).group(0)
                print re.search(programming_regex4, cve.description)#.group(0)
            result.append(config.Vulnerability.Programming_Errors)
        if re.search(web, cve.description):
            result.append(config.Vulnerability.Web)
        if (re.search(weak_ac, cve.description) or re.search(weak_ac2, cve.description)
            or re.search(weak_ac3, cve.description)) \
                and not re.search(web, cve.description):
            if cve.id == 'CVE-2013-1218':
                print re.search(weak_ac, cve.description)
                print re.search(weak_ac2, cve.description)#.group(0)
                print re.search(weak_ac3, cve.description).group(0)
            result.append(config.Vulnerability.Weak_Access_Control)
        if re.search(improper_crypto,cve.description):
            result.append(config.Vulnerability.Improper_Crypto)
        if re.search(insecure_config,cve.description) \
                and not (re.search(weak_ac, cve.description) or re.search(weak_ac2, cve.description)):
            if cve.id == 'CVE-2007-4926':
                print re.search(insecure_config,cve.description).group(0)
            result.append(config.Vulnerability.Insecure_Config)
    if len(result) == 0:
        print cve.id, cve.description
        print 'PARAAAAA - VULN?'
    return result